@@ -1,6 +1,6 @@
 from pathlib import Path
 from contextlib import contextmanager
-
+import warnings
 import numpy as np
 from ._version import get_versions
 
@@ -18,9 +18,68 @@ __all__ = ['RaggedArray', 'asraggedarray', 'create_raggedarray',
 # TODO needs doc
 # TODO an open_array method
 class RaggedArray:
-    """
-    Disk-based sequence of arrays that may have a variable length in maximally
-    one dimension.
+    """Instantiate a Darr ragged array from disk.
+
+    A ragged array is a sequence of subarrays that may be multidimesional,
+    with the restriction that their dimensional shape is the same except for
+    their first axis. In the simplest case it is a sequence of variable-length
+    one-dimensional subarrays, e.g.::
+
+        [[1,2],
+         [3,4,5],
+         [6],
+         [7,8,9,10]]
+
+    , but the subarrays can also be multidimensional, e.g.::
+
+        [[[1,2],[3,4]],
+         [[5,6],[7,8],[9,10]],
+         [[11,12]],
+         [[13,14],[15,16]]]
+
+    In the latter case they are they are two-dimensional, and the length of
+    their second axis is fixed: length 2. The atom shape of the array is
+    said to be (2,). If subarrays were four-dimensional their atom shape
+    could be, e.g. (2,7,5), and their dimensionality (N,2,7,5), where N is
+    an integer. In Darr N can also be zero.
+
+    Ragged arrays are often used for time series data that has been
+    collected in multiple episodes of varying duration, although other use
+    cases exist.
+
+    On disk, a Darr ragged array corresponds to a directory containing 1) a
+    Darr array called 'values' in which all subarrays have been concatenated
+    along their first dimension. 2) a Darr array called 'indices', which is
+    two-dimensional an hold the indices to obtain the subarrays from the values
+    array. 3) a text file (JSON format) describing the numeric type, array size
+    and length, and other format information, and 4) a README text file
+    documenting the data format, including code to read the array data in other
+    programming languages.
+
+    A RaggedArray can be indexed with an integer to get the subarrays as NumPy
+    arrays.
+
+    Parameters
+    ----------
+    path : str or pathlib.Path
+        Path to disk-based array directory.
+    accessmode : {'r', 'r+'}, default 'r'
+       File access mode of the darr data. `r` means read-only, `r+` means
+       read-write. `w` does not exist. To create new darr arrays, potentially
+       overwriting an other one, use the `asarray` or `create_array`
+       functions.
+
+    Examples
+    --------
+    >>> import darr
+    >>> ra = darr.asraggedarray('test.darr', [[1,2],[1,2,3,4],[5,6,7]], dtype='int32')
+    >>> ra
+    RaggedArray (3 subarrays with atom shape (), r+)
+    >>> del ra
+    >>> ra = darr.RaggedArray('test.darr')
+    >>> ra[1] # will return a NumPy array
+    array([1, 2, 3, 4], dtype=int32)
+
 
     """
     _valuesdirname = 'values'
@@ -90,7 +149,7 @@ class RaggedArray:
 
     @property
     def narrays(self):
-        """Numpy data type of the array values.
+        """number of subarrays in the RaggedArray.
 
         """
         return self._indices.shape[0]
@@ -117,14 +176,14 @@ class RaggedArray:
 
     @property
     def size(self):
-        """Total number of values in the data array."""
+        """Total number of values in the ragged array."""
         return int(self._values._size)
 
     @property
     def readcodelanguages(self):
         """Tuple of the languages that the `readcode` method can produce
-                reading code for. Code in these languages is also included in the
-                README.txt file that is stored as part of the array ."""
+        reading code for. Code in these languages is also included in the
+        README.txt file that is stored as part of the array ."""
         languages = []
         for lang in readcodefunc.keys():
             if readcode(self, lang) is not None:
@@ -156,37 +215,116 @@ class RaggedArray:
         self._datadir._write_jsondict(filename=self._arraydescrfilename,
                                       d=self._arrayinfo, overwrite=True)
 
-    def _append(self, array):
-        size = len(array)
-        endindex = self._values.shape[0]
-        self._values.append(np.asarray(array, dtype=self.dtype))
-        self._indices.append([[endindex, endindex + size]])
+    # TODO this can be made more efficient by using _append on self._values
+    #  and self._indices and returning length increases
 
+    def _append(self, array, fdv, fdi, vlen):
+        size = len(array)
+        #endindex = self._values._memmap.shape[0]
+        vlenincr = self._values._append(np.asarray(array, dtype=self.dtype),
+                                        fdv)
+        ilenincr = self._indices._append([[vlen, vlen + size]], fdi)
+        return (vlenincr, ilenincr)
 
     def append(self, array):
-        self._append(array)
-        self._update_readmetxt()
-        self._update_arraydescr(len=len(self._indices),
-                                size=self._values.size)
+        """Append array-like objects to the ragged array.
 
-    def copy(self, path, accessmode='r', overwrite=False):
+        The shape of the data and the darr must be compliant. The length of
+        its first axis may vary, but if the are more axes, these should
+        have the same lengths as all other subarrays (which is the `atom` of
+        the raged array). When appending data repeatedly it is more efficient
+        to use `iterappend`.
+
+
+        Parameters
+        ----------
+        array: array-like object
+            This can be a numpy array, a sequence that can be converted into a
+            numpy array.
+
+        Returns
+        -------
+            None
+
+        """
+        with self.open_arrays() as ((iv, vv), (fdv, fdi)):
+            vlen = self._values.shape[0]
+            vlenincr, ilenincr = self._append(array, fdv, fdi, vlen)
+            self._values._update_len(lenincrease=vlenincr)
+            self._indices._update_len(lenincrease=ilenincr)
+            self._update_readmetxt()
+            self._update_arraydescr(len=len(self._indices),
+                                    size=self._values.size)
+
+    def copy(self, path, dtype=None, accessmode='r', overwrite=False):
+        """Copy darr to a different path, potentially changing its dtype.
+
+        The copying is performed in chunks to avoid RAM memory overflow for
+        very large darr arrays.
+
+        Parameters
+        ----------
+        path: str or pathlib.Path
+        dtype: <dtype, None>
+            Numpy data type of the copy. Default is None, which corresponds to
+            the dtype of the darr to be copied.
+        accessmode: {'r', 'r+'}, default 'r'
+            File access mode of the darr data of the returned Darr
+            object. `r` means read-only, `r+` means read-write.
+        overwrite: (True, False), optional
+            Overwrites existing darr data if it exists. Note that a
+            darr path is a directory. If that directory contains
+            additional files, these will not be removed and an OSError is
+            raised. Default is `False`.
+
+        Returns
+        -------
+        Array
+           copy of the darr array
+
+        """
         arrayiterable = (self[i] for i in range(len(self)))
         metadata = dict(self.metadata)
+        if dtype is None:
+            dtype = self.dtype
         return asraggedarray(path=path, arrayiterable=arrayiterable,
-                             dtype=self.dtype, metadata=metadata,
+                             dtype=dtype, metadata=metadata,
                              accessmode=accessmode, overwrite=overwrite)
 
     @contextmanager
     def _view(self, accessmode=None):
+        warnings.warn("The use of the `_view` method is deprecated in "
+                      "versions of Darr >= 0.6 Use `open_arrays` instead.",
+                      FutureWarning)
         with self._indices._open_array(accessmode=accessmode) as (iv, _), \
              self._values._open_array(accessmode=accessmode) as (vv, _):
             yield iv, vv
 
+    @contextmanager
+    def open_arrays(self, accessmode=None):
+        with self._indices._open_array(accessmode=accessmode) as (iv, fdi), \
+                self._values._open_array(accessmode=accessmode) as (vv, fdv):
+            yield (iv, vv), (fdv, fdi)
+
     def iter_arrays(self, startindex=0, endindex=None, stepsize=1,
                  accessmode=None):
+        """Iterate over ragged array yielding subarrays.
+
+        startindex: <int, None>
+            Start index value.
+            Default is None, which means to start at the beginning.
+        endindex: <int, None>
+            End index value.
+            Default is None, which means to end at the end.
+        stepsize: <int, None>
+            Size of the shift per iteration across the first axis.
+            Default is None, which means that `stepsize` equals `chunklen`.
+
+        """
+
         if endindex is None:
             endindex = self.narrays
-        with self._view(accessmode=accessmode):
+        with self.open_arrays(accessmode=accessmode):
             for i in range(startindex, endindex, stepsize):
                 yield np.array(self[i], copy=True)
 
@@ -206,13 +344,20 @@ class RaggedArray:
             None
 
         """
-        # TODO refactor such that info files are not updated at each append?
-        with self._view():
+
+        with self.open_arrays() as ((iv, vv), (fdv, fdi)):
+            vlenincr = 0
+            ilenincr = 0
+            vlen = self._values.shape[0]
             for a in arrayiterable:
-                self._append(a)
-        self._update_readmetxt()
+                vli, ili = self._append(a, fdv, fdi, vlen+vlenincr)
+                vlenincr += vli
+                ilenincr += ili
+        self._values._update_len(lenincrease=vlenincr)
+        self._indices._update_len(lenincrease=ilenincr)
         self._update_arraydescr(len=len(self._indices),
                                 size=self._values.size)
+        self._update_readmetxt()
 
     def readcode(self, language, abspath=False, basepath=None):
         """Generate code to read the array in a different language.
@@ -255,7 +400,7 @@ class RaggedArray:
         return readcode(self, language, basepath=basepath, abspath=abspath)
 
     def archive(self, filepath=None, compressiontype='xz', overwrite=False):
-        """Archive array data into a single compressed file.
+        """Archive ragged array data into a single compressed file.
 
         Parameters
         ----------
@@ -288,9 +433,46 @@ class RaggedArray:
 
 
 # FIXME empty arrayiterable
-# TODO describe int32 vs int64 index
 def asraggedarray(path, arrayiterable, dtype=None, metadata=None,
                   accessmode='r+', indextype='int64', overwrite=False):
+    """Creates an empty RaggedArray.
+
+        Parameters
+        ----------
+        path : str or pathlib.Path
+            Path to disk-based array directory.
+        arrayiterable: iterator yielding array-like objects
+            This can be a numpy array, a sequence that can be converted into a
+            numpy array, or an iterator that yields such objects. The latter
+            will be concatenated along the first dimension.
+        dtype : dtype, optional
+            The type of the `darr`. Default is 'float64'
+        metadata: {None, dict}
+            Dictionary with metadata to be saved in a separate JSON file. Default
+            None
+        accessmode : {'r', 'r+'}, default 'r'
+           File access mode of the darr data. `r` means read-only, `r+` means
+           read-write. `w` does not exist. To create new darr arrays, potentially
+           overwriting an other one, use the `asarray` or `create_array`
+           functions.
+        indextype : dtype, optional
+            The dtype of the index array underlying the disk-based ragged array
+            format. Defaults to 'int64'. But other possibilities are int8','uint8',
+            'int16', 'uint16', 'int32', 'uint32', 'int64'. This determines the
+            maximum length of the ragged array, but also how compatible the
+            array is with other languages.
+        overwrite: <True, False>, optional
+            Overwrites existing darr data if it exists. Note that a darr
+            paths is a directory. If that directory contains additional files,
+            these will not be removed and an OSError is raised.
+            Default is `False`.
+
+        Returns
+        -------
+        RaggedArray
+            A Darr RaggedArray instance.
+
+        """
     path = Path(path)
     supportedindextypes = ('int8','uint8', 'int16', 'uint16', 'int32',
                            'uint32', 'int64')
@@ -346,6 +528,45 @@ def asraggedarray(path, arrayiterable, dtype=None, metadata=None,
 
 def create_raggedarray(path, atom=(), dtype='float64', metadata=None,
                        accessmode='r+', indextype='int64', overwrite=False):
+    """Creates an empty RaggedArray.
+
+    Parameters
+    ----------
+    path : str or pathlib.Path
+        Path to disk-based array directory.
+    atom: tuple
+        shape of the subarray dimensions, except the first dimension.
+        Default is (), meaning that the ragged array consists of
+        one-dimensional subarrays. If subarrays, e.g., have a second and
+        third dimension of length 4 and 7, the atom would be (4,7).
+    dtype : dtype, optional
+        The type of the `darr`. Default is 'float64'
+    metadata: {None, dict}
+        Dictionary with metadata to be saved in a separate JSON file. Default
+        None
+    accessmode : {'r', 'r+'}, default 'r'
+       File access mode of the darr data. `r` means read-only, `r+` means
+       read-write. `w` does not exist. To create new darr arrays, potentially
+       overwriting an other one, use the `asarray` or `create_array`
+       functions.
+    indextype : dtype, optional
+        The dtype of the index array underlying the disk-based ragged array
+        format. Defaults to 'int64'. But other possibilities are int8','uint8',
+        'int16', 'uint16', 'int32', 'uint32', 'int64'. This determines the
+        maximum length of the ragged array, but also how compatible the
+        array is with other languages.
+    overwrite: <True, False>, optional
+        Overwrites existing darr data if it exists. Note that a darr
+        paths is a directory. If that directory contains additional files,
+        these will not be removed and an OSError is raised.
+        Default is `False`.
+
+    Returns
+    -------
+    RaggedArray
+        A Darr RaggedArray instance.
+
+    """
     if not hasattr(atom, '__len__'):
         raise TypeError(f'shape "{atom}" is not a sequence of dimensions.\n'
                         f'If you want just a list of 1-dimensional arrays, '
@@ -363,7 +584,6 @@ def create_raggedarray(path, atom=(), dtype='float64', metadata=None,
     return RaggedArray(ra.path, accessmode=accessmode)
 
 # TODO, simplify explanation if subarrays are 1-dimensional
-# TODO add readcode for more languages
 def readmetxt(ra):
     n = len(ra)
     ndsa  = len(ra.atom)
@@ -476,7 +696,7 @@ def delete_raggedarray(ra):
 
     Parameters
     ----------
-    path: path to data directory
+    ra: RaggedArray or path to RaggedArray to be deleted.
 
     """
     try:
